package com.sun.security.sasl;

import javax.security.auth.callback.UnsupportedCallbackException;
import java.io.IOException;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.NameCallback;
import javax.security.sasl.SaslException;
import com.sun.security.sasl.util.PolicyUtils;
import javax.security.sasl.SaslClient;
import javax.security.auth.callback.CallbackHandler;
import java.util.Map;
import javax.security.sasl.SaslClientFactory;

public final class ClientFactoryImpl implements SaslClientFactory
{
    private static final String[] myMechs;
    private static final int[] mechPolicies;
    private static final int EXTERNAL = 0;
    private static final int CRAMMD5 = 1;
    private static final int PLAIN = 2;
    
    public ClientFactoryImpl() {
    }
    
    @Override
    public SaslClient createSaslClient(String[] array, String s, String s2, String s3, Map<String, ?> props, CallbackHandler callbackHandler) throws SaslException {
        int i = 0;
        while (i < array.length) {
            if (array[i].equals(ClientFactoryImpl.myMechs[0]) && PolicyUtils.checkPolicy(ClientFactoryImpl.mechPolicies[0], props)) {
                return new ExternalClient(s);
            }
            if (array[i].equals(ClientFactoryImpl.myMechs[1]) && PolicyUtils.checkPolicy(ClientFactoryImpl.mechPolicies[1], props)) {
                Object[] userInfo2 = this.getUserInfo("CRAM-MD5", s, callbackHandler);
                return new CramMD5Client((String)userInfo2[0], (byte[])(byte[])userInfo2[1]);
            }
            if (array[i].equals(ClientFactoryImpl.myMechs[2]) && PolicyUtils.checkPolicy(ClientFactoryImpl.mechPolicies[2], props)) {
                Object[] userInfo = this.getUserInfo("PLAIN", s, callbackHandler);
                return new PlainClient(s, (String)userInfo[0], (byte[])(byte[])userInfo[1]);
            }
            ++i;
        }
        return null;
    }
    
    @Override
    public String[] getMechanismNames(Map<String, ?> props) {
        return PolicyUtils.filterMechs(ClientFactoryImpl.myMechs, ClientFactoryImpl.mechPolicies, props);
    }
    
    private Object[] getUserInfo(String s, String defaultName, CallbackHandler callbackHandler) throws SaslException {
        if (callbackHandler == null) {
            throw new SaslException("Callback handler to get username/password required");
        }
        try {
            String string = s + " authentication id: ";
            String string2 = s + " password: ";
            NameCallback nameCallback = (defaultName == null) ? new NameCallback(string) : new NameCallback(string, defaultName);
            PasswordCallback passwordCallback = new PasswordCallback(string2, false);
            callbackHandler.handle(new Callback[] { nameCallback, passwordCallback });
            char[] password = passwordCallback.getPassword();
            byte[] bytes;
            if (password != null) {
                bytes = new String(password).getBytes("UTF8");
                passwordCallback.clearPassword();
            }
            else {
                bytes = null;
            }
            return new Object[] { nameCallback.getName(), bytes };
        }
        catch (IOException ex) {
            throw new SaslException("Cannot get password", ex);
        }
        catch (UnsupportedCallbackException ex2) {
            throw new SaslException("Cannot get userid/password", ex2);
        }
    }
    
    static {
        myMechs = new String[] { "EXTERNAL", "CRAM-MD5", "PLAIN" };
        mechPolicies = new int[] { 7, 17, 16 };
    }
}
